; CreativeBoard					(c) 2015 Cygnus2
; www.cygnus2.net
;
;	**********************************************************************************************************
;	**													**
;	**				S O U R C E    C O D E    R E V I S I O N				**
;	**													**
;	**********************************************************************************************************
;	**													**
;	** 					Created by:  Vladimir RYKL					**
;	**													**
;	**********************************************************************************************************
;	**	   *		*										**
;	** Version *    Date 	*				Description					**
;	** 	   *		*										**
;	**********************************************************************************************************
;	**  V0.01  * 18/02/2015	* First release	- MCU PIC16F630	used instead of PIC16F505			**
;	** 	   *		*										**
;	** 	   *		* ROM occupation: 96.1% (984 of 1024 words used)				**
;	**********************************************************************************************************
;	**  V1.00  * 09/06/2015	* Adapted to the use of MCU PIC16F610, otherwise the same as V0.01		**
;	** 	   *		*										**
;	** 	   *		* ROM occupation: 96.0% (983 of 1024 words used)				**
;	**********************************************************************************************************
;
	list      p=16F610		; list directive to define processor
	#include <p16F610.inc>		; processor specific variable definitions
;
;************************************************************************************************************************
;		R E G I S T E R     A N D     B I T     D E F I N I T I O N
;************************************************************************************************************************
; Program control register 1
reg_ProgCtrl1		EQU	07FH
#define		bit_MicIn		reg_ProgCtrl1,007H	; Microphone input active
#define		bit_Beep		reg_ProgCtrl1,006H	; Beep request
#define		bit_SoundOn		reg_ProgCtrl1,005H	; Sound ON/OFF (0 - sound off, 1 - sound on)
#define		bit_Div2		reg_ProgCtrl1,004H	; Tone generation auxiliary bit (frequency divide by two)
#define		bit_Freq		reg_ProgCtrl1,003H	; Tone generation auxiliary bit
#define		bit_Tone2		reg_ProgCtrl1,002H	; Tone 2 flag (0 - tone off, 1 - tone on)
#define		bit_Tone1		reg_ProgCtrl1,001H	; Tone 1 flag (0 - tone off, 1 - tone on)
#define		bit_Key			reg_ProgCtrl1,000H	; Key flag (0 - all keys released, 1 - any key pressed)
; Program control register 2
reg_ProgCtrl2		EQU	07EH
#define		bit_Function09		reg_ProgCtrl2,002H	; Function 09 active
#define		bit_Function08		reg_ProgCtrl2,001H	; Function 08 active
#define		bit_Function06		reg_ProgCtrl2,000H	; Function 06 active
; Delay control register
reg_DelayCtrl		EQU	07DH				; Delay expiration flags
#define		bit_LongDelay2		reg_DelayCtrl,004H	; Long delay 2 expiration flag
#define		bit_LongDelay1		reg_DelayCtrl,003H	; Long delay 1 expiration flag
#define		bit_ShortDelay2		reg_DelayCtrl,002H	; Short delay 2 expiration flag
#define		bit_ShortDelay1		reg_DelayCtrl,001H	; Short delay 1 expiration flag
#define		bit_BasicDelay		reg_DelayCtrl,000H	; Basic delay expiration flag
; Delay counter registers
reg_LongDelay2H		EQU	07CH				; Long delay counter register (high & low byte)
reg_LongDelay2L		EQU	07BH				; //
reg_LongDelay1H		EQU	07AH				; //
reg_LongDelay1L		EQU	079H				; //
reg_ShortDelay2		EQU	078H				; Short delay counter register
reg_ShortDelay1		EQU	077H				; //
; LED driver
reg_LEDDriver		EQU	076H				; //
; Key (switch) registers
reg_KeyCode		EQU	075H				; Key code register
reg_KeyCdStk		EQU	074H				; Key code stack
reg_KeySamp		EQU	073H				; Key sampling register
; Function phase registers
reg_F03Phase		EQU	072H				; Function phase registers
reg_F04Phase		EQU	071H				; //
reg_F06Phase		EQU	070H				; //
reg_F07Phase		EQU	06FH				; //
reg_F08Phase		EQU	06EH				; //
reg_F09Phase		EQU	06DH				; //
; Temporary registers
reg_Temp1		EQU	06CH				; //
; Pseudo-random value generator
reg_RND1		EQU	06BH				; Pseudo-random accumulator
reg_RND2		EQU	06AH				; //
reg_RND3		EQU	069H				; //
reg_RND4		EQU	068H				; //
reg_Random		EQU	067H				; Result register
;
reg_Config		EQU	066H				; MODE selector
reg_BeepCode		EQU	065H				; Beep code
; Light effect register set
reg_EffectCode		EQU	064H				; Registers shared with game register set
reg_EffectSpeed		EQU	063H				; //
reg_EffectLength	EQU	062H				; //
reg_EffectPtr		EQU	061H				; //
; Game 1 & 2 register set
reg_GameLevel		EQU	064H				; Registers shared with light effect register set
reg_GameSpeed		EQU	063H				; //
reg_GameStep		EQU	062H				; //
reg_GameRnd		EQU	061H				; //
reg_GameRepeat		EQU	060H				; //
reg_GameBuf1		EQU	05FH				; //
reg_GameBuf2		EQU	05EH				; //
;
;************************************************************************************************************************
;		P O R T     D E F I N I T I O N
;************************************************************************************************************************
;
#define		port_ROW1		PORTC,005H		; Row 1 driver
#define		port_ROW2		PORTC,004H		; Row 2 driver
#define		port_COL1		PORTC,000H		; Column 1 driver
#define		port_COL2		PORTC,001H		; Column 2 driver
#define		port_COL3		PORTC,003H		; Column 3 driver
#define		port_COL4		PORTC,002H		; Column 4 driver
#define		port_CFG1		PORTA,000H		; Configuration input 1
#define		port_CFG2		PORTA,001H		; Configuration input 2
#define		port_CFG3		PORTA,004H		; Configuration input 3
#define		port_BUZ		PORTA,005H		; Buzzer driver
#define		port_MIC		PORTA,002H		; Microphone input
;
;************************************************************************************************************************
;		C O N S T A N T     D E F I N I T I O N
;************************************************************************************************************************
;
con_BuzzerMask		EQU	020H				; XOR mask for sound generation
con_Game1MaxLevel	EQU	004H				; Game 1 maximum level
con_Game2MaxLevel	EQU	006H				; Game 2 maximum level
;
;************************************************************************************************************************
;		P I C 1 6 F 6 3 0     C O N F I G U R A T I O N     F U S E S
;************************************************************************************************************************
;
;	__CONFIG   _CP & _CPD & _BODEN_OFF & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT
	__CONFIG   _BOD_OFF & _IOSCFS_4MHZ & _CP_OFF & _MCLRE_OFF & _PWRTE_ON & _WDT_ON  & _INTRC_OSC_NOCLKOUT
;
; MCLR configured as I/O
;
;************************************************************************************************************************
;*		A P P L I C A T I O N     B E G I N									*
;************************************************************************************************************************
;
		ORG	0000H				; RESET vector
;
sa_RESET:	MOVLW	0FFH				; Initialize port driver
		MOVWF	PORTA				; //
		CLRF	PORTC				; //
		GOTO	sa_INIT				; Skip interrupt service routine
;
;************************************************************************************************************************
;*		I N T E R R U P T     S U B R O U T I N E S								*
;************************************************************************************************************************
;
		ORG	0004H				; Interrupt vector
; TMR0 interrupt service routine - TMR0 interrupt occurs every 256 usec
		BCF	INTCON,T0IF			; Clear TMR0 interrupt flag
		BTFSC	bit_Tone1			; Tone 1 (2kHz) requested ?
		GOTO	sa_TONE10			; // YES !
		BTFSC	bit_Tone2			; Tone 2 (1kHz) requested ?
		GOTO	sa_TONE20			; // YES !
		GOTO	sa_DELAY			; // NO ! - skip tone generation
;
sa_TONE20:	BTFSS	bit_Div2			; Toggle auxiliary bit_Div2 --> tone frequency / 2 (1 kHz)
		GOTO	sa_TONE21			; //
		BCF	bit_Div2			; //
		GOTO	sa_DELAY			; //
sa_TONE21:	BSF	bit_Div2			; //
;
sa_TONE10:	BTFSS	bit_Freq			; Toggle buzzer port and auxiliary bit_Freq
		GOTO	sa_TONE11			; //
		BCF	bit_Freq			; //
		BCF	port_BUZ			; //
		GOTO	sa_DELAY			; //
sa_TONE11:	BSF	bit_Freq			; //
		BSF	port_BUZ			; //
; Basic delay
sa_DELAY:	BSF	bit_BasicDelay			; Set basic delay expiration flag
; Short delay 1 (watchdog reset)
		DECFSZ	reg_ShortDelay1,F		; Delay expired ?
		GOTO	sa_TMR0A			; // NO !
		BSF	bit_ShortDelay1			; // YES ! - set expiration flag
; Short delay 2 (key samplig period)
sa_TMR0A:	DECFSZ	reg_ShortDelay2,F		; Delay expired ?
		GOTO	sa_TMR0B			; // NO !
		BSF	bit_ShortDelay2			; // YES ! - set expiration flag
; Long delay 1 (beep duration)
sa_TMR0B:	DECFSZ	reg_LongDelay1L,F		; Delay expired ?
		GOTO	sa_TMR0C			; // NO !
		DECFSZ	reg_LongDelay1H,F		; //
		GOTO	sa_TMR0C			; // NO !
		BSF	bit_LongDelay1			; // YES ! - set expiration flag
; Long delay 2 (light effect speed / game 1 speed)
sa_TMR0C:	DECFSZ	reg_LongDelay2L,F		; Delay expired ?
		GOTO	sa_TMR0D			; // NO !
		DECFSZ	reg_LongDelay2H,F		; //
		GOTO	sa_TMR0D			; // NO !
		BSF	bit_LongDelay2			; // YES ! - set expiration flag
sa_TMR0D:	RETFIE					; Exit interrupt service routine
;
;************************************************************************************************************************
;*		S Y S T E M     I N I T I A L I Z A T I O N								*
;************************************************************************************************************************
;
;		---------------------
;		HARDWARE MODULE SETUP
;		---------------------
;
; I/O port setup
sa_INIT:	BSF	STATUS,RP0			; --> REGISTER BANK 1
		MOVLW	017H				; RA0, RA1, RA2, RA4 = input, RA3, RA5 = output
		MOVWF	TRISA				; //
		MOVLW	013H				; Active pull-ups @ RA0	& RA1 & RA4
		MOVWF	WPUA				; //
		MOVLW	000H				; RC0 - RC5 = output
		MOVWF	TRISC				; //
		CLRF	ANSEL				; //
; TMR0 setup, clock source Fosc/4 ~ 1 MHz, without prescaler)
		MOVLW	05BH				; WDT prescaler 1:8, internal pull-ups active
		MOVWF	OPTION_REG			; //
		BCF	STATUS,RP0			; --> REGISTER BANK 0
		CLRF	TMR0				; Clear TMR0
;
;		---------------------------
;		MCU REGISTER INITIALIZATION
;		---------------------------
;
		CLRF	reg_ProgCtrl1			; Clear program control registers
		CLRF	reg_ProgCtrl2			; //
		MOVLW	0FFH				; Set all delay expiration flags
		MOVWF	reg_DelayCtrl			; //
		MOVLW	0FFH				; Initialize MODE selector with invalid value to ensure
		MOVWF	reg_Config			; // appropriate configuration after RESET(fuction 05)
		CLRF	reg_KeyCode			; Clear key code register
		CLRF	reg_KeyCdStk			; Clear key code stack
		MOVLW	055H				; Initialize key sampling register
		MOVWF	reg_KeySamp			; //
		CLRF	reg_F03Phase			; Clear all function phase registers
		CLRF	reg_F04Phase			; //
		CLRF	reg_F06Phase			; //
		CLRF	reg_F07Phase			; //
		CLRF	reg_F08Phase			; //
		CLRF	reg_F09Phase			; //
		CLRF	reg_LEDDriver			; Initialize LED driver (all LEDs turned off)
		CLRF	reg_RND1			; Clear pseudo-random generator
		CLRF	reg_RND2			; //
		CLRF	reg_RND3			; //
		CLRF	reg_RND4			; //
		BSF	INTCON,T0IE			; TMR0 interrupt enable
		BSF	INTCON,GIE			; Global interrupt enable
;
;************************************************************************************************************************
;*		M A I N     P R O G R A M     L O O P									*
;************************************************************************************************************************
;
sa_MAINLOOP
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 01: Watchdog reset every 56 msec with 1:8 prescaler (basic watchdog time-out period Twdt = 10 - 30 msec)
		BTFSS	bit_ShortDelay1			; Watchdog reset delay expired ?
		GOTO	sa_F01EXIT			; // NO ! - exit function
		BCF	bit_ShortDelay1			; // YES ! - clear expiration flag
		MOVLW	075H				; // Reinitialize delay for 30 msec
		MOVWF	reg_ShortDelay1			; //
		CLRWDT					; Reset watchdog timer
;------------------------------------------------------------------------------------------------------------------------
sa_F01EXIT						; FUNCTION 01 delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 02: pseudo-random generator (values 0x00, 0x01, 0x02 and 0x03) - 0x64/0x86/0x9B/0x57
		MOVLW	014H				; Generate pseudo-random number
		ADDWF	reg_RND1,F			; //
		MOVLW	082H				; //
		ADDWF	reg_RND2,F			; //
		MOVLW	03CH				; //
		ADDWF	reg_RND3,F			; //
		MOVLW	001H				; //
		ADDWF	reg_RND4,F			; //
		MOVF	reg_RND1,W			; //
		XORWF	reg_RND2,W			; //
		XORWF	reg_RND3,W			; //
		XORWF	reg_RND4,W			; //
		ANDLW	030H				; //
		MOVWF	reg_Random			; Store pseudo-random value
		SWAPF	reg_Random,F			; //
;------------------------------------------------------------------------------------------------------------------------
sa_F02EXIT						; FUNCTION 06 delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 03: LED driver control & key sampling - 8 subsequent samples have to be equal to recognize valid key code
		BTFSS	bit_BasicDelay			; Row switching delay expired ?
		GOTO	sa_F03EXIT			; // NO ! - exit function
		BCF	bit_BasicDelay			; // YES ! - clear expiration flag
;
		MOVLW	high(sa_F03PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg_F03Phase,W			; //
		ADDWF	PCL,F				; //
sa_F03PHASE:	GOTO	sa_F03PHASE0			; //
sa_F03PHEND:	GOTO	sa_F03PHASE1			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F03PHASE) - high(sa_F03PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F03PHASE0:	BCF	port_ROW2			; Turn off ROW2 driver before turning on ROW1 driver
		BTFSC	reg_LEDDriver,000H		; reg_LEDDriver -> ports COL1 - COL4
		BSF	port_COL1			; Bit no. 0
		BTFSS	reg_LEDDriver,000H		; //
		BCF	port_COL1			; //
		BTFSC	reg_LEDDriver,001H		; Bit no. 1
		BSF	port_COL2			; //
		BTFSS	reg_LEDDriver,001H		; //
		BCF	port_COL2			; //
		BTFSC	reg_LEDDriver,002H		; Bit no. 2
		BSF	port_COL3			; //
		BTFSS	reg_LEDDriver,002H		; //
		BCF	port_COL3			; //
		BTFSC	reg_LEDDriver,003H		; Bit no. 3
		BSF	port_COL4			; //
		BTFSS	reg_LEDDriver,003H		; //
		BCF	port_COL4			; //
		BSF	port_ROW1			; Turn on ROW1 driver
		INCF	reg_F03Phase,F			; Phase 1
		GOTO	sa_F03EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F03PHASE1:	BCF	port_ROW1			; Turn off ROW1 driver before turning on ROW2 driver
		BTFSC	reg_LEDDriver,004H		; reg_LEDDriver -> ports COL1 - COL4
		BSF	port_COL1			; Bit no. 4
		BTFSS	reg_LEDDriver,004H		; //
		BCF	port_COL1			; //
		BTFSC	reg_LEDDriver,005H		; Bit no. 5
		BSF	port_COL2			; //
		BTFSS	reg_LEDDriver,005H		; //
		BCF	port_COL2			; //
		BTFSC	reg_LEDDriver,006H		; Bit no. 6
		BSF	port_COL3			; //
		BTFSS	reg_LEDDriver,006H		; //
		BCF	port_COL3			; //
		BTFSC	reg_LEDDriver,007H		; Bit no. 7
		BSF	port_COL4			; //
		BTFSS	reg_LEDDriver,007H		; //
		BCF	port_COL4			; //
;
		BTFSS	bit_ShortDelay2			; Key sampling period expired ?
		GOTO	sa_F03PH190			; // NO ! - skip key sampling
		BCF	bit_ShortDelay2			; // YES ! - clear expiration flag
		MOVLW	007H				; Reinitialize delay for 2 msec
		MOVWF	reg_ShortDelay2			; //
;
		BSF	STATUS,RP0			; --> REGISTER BANK 1
		MOVLW	00FH				; Configure RC0 & RC1 & RC2 & RC3 as input to read key status
		MOVWF	TRISC				; //
		BCF	STATUS,RP0			; --> REGISTER BANK 0
;
		COMF	PORTC,W				; Store current key status into temporary register
		ANDLW	00FH				; //
		MOVWF	reg_Temp1			; //
;
		BSF	STATUS,RP0			; --> REGISTER BANK 1
		CLRF	TRISC				; Reconfigure RC as output to drive LEDs
		BCF	STATUS,RP0			; --> REGISTER BANK 0
;
		XORWF	reg_KeyCdStk,W			; Compare two consecutive key code samples
		BCF	STATUS,C			; If two consecutive samples equal, shift in '0', otherwise	
		BTFSS	STATUS,Z			; // shift in '1'
		BSF	STATUS,C			; //
		RLF	reg_KeySamp,F			; //
;
		MOVF	reg_Temp1,W			; Restore current key status from temporary register
		MOVF	reg_KeySamp,F			; Last 8 samples equal ?
		BTFSC	STATUS,Z			; //
		MOVWF	reg_KeyCode			; // YES ! - update key code with new value
		MOVWF	reg_KeyCdStk			; //
;
		MOVLW	000H				; reg_KeyCode = 0x00 --> bit_Key = 0
		XORWF	reg_KeyCode,W			; reg_KeyCode <> 0x00 --> bit_Key = 1
		BTFSC	STATUS,Z			; //
		BCF	bit_Key				; //
		BTFSS	STATUS,Z			; //
		BSF	bit_Key				; //
;
sa_F03PH190:	BSF	port_ROW2			; Turn on ROW2 driver
		CLRF	reg_F03Phase			; Phase 0
		GOTO	sa_F03EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F03EXIT						; FUNCTION 03 delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 04: Sound generator - maximum beep duration about 4 sec
;	|-----------|-----------|------------------|
;	| bit_Tone2 | bit_Tone1 | output frequency |
;	|-----------|-----------|------------------|
;	|     0     |     0     |       0 kHz      |
;	|     0     |     1     |       2 kHz      |
;	|     1     |     0     |       1 kHz      |
;	|     1     |     1     |    not allowed   |
;	|-----------|-----------|------------------|
;
		BTFSS	bit_Beep			; Beep requested ?
		GOTO	sa_F04EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F04PHASE)		; // YES ! - function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg_F04Phase,W			; //
		ADDWF	PCL,F				; //
sa_F04PHASE:	GOTO	sa_F04PHASE0			; //
sa_F04PHEND:	GOTO	sa_F04PHASE1			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F04PHASE) - high(sa_F04PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F04PHASE0:	CALL	sa_F04TABLE			; Beep decoding
		MOVWF	reg_Temp1			; //
		ANDLW	03FH				; Initialize beep duration
		MOVWF	reg_LongDelay1H			; //
		MOVLW	01FH				; //
		MOVWF	reg_LongDelay1L			; //
		BCF	bit_LongDelay1			; Clear expiration flag
;
		INCF	reg_F04Phase,F			; Phase 1
		BTFSS	bit_SoundOn			; Sound active ?
		GOTO	sa_F04EXIT			; // NO ! - exit function
		RLF	reg_Temp1,F			; // YES ! - tone frequency decoding
		BTFSS	STATUS,C			; //
		BCF	bit_Tone2			; //
		BTFSC	STATUS,C			; //
		BSF	bit_Tone2			; // 1 kHz
		RLF	reg_Temp1,F			; //
		BTFSS	STATUS,C			; //
		BCF	bit_Tone1			; //
		BTFSC	STATUS,C			; //
		BSF	bit_Tone1			; // 2 kHz
		GOTO	sa_F04EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F04PHASE1:	BTFSS	bit_LongDelay1			; Beep duration expired ?
		GOTO	sa_F04EXIT			; // NO !
		BSF	port_BUZ			; // YES ! - set buzzer driver high to reduce current consumption
		BCF	bit_Tone1			; Stop tone generation
		BCF	bit_Tone2			; //
		BCF	bit_Beep			; Stop function 04
		CLRF	reg_F04Phase			; Phase 0
		GOTO	sa_F04EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F04TABLE:	MOVLW	high(sa_F04TABBEG)		; Beep duration
		MOVWF	PCLATH				; //
		MOVF	reg_BeepCode,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F04TABBEG) - high(sa_F04TABEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; Beep encoding
;	|  7. bit   |  6. bit   |  5. bit   |  4. bit   |  3. bit   |  2. bit   |  1. bit   |  0. bit   |
;	| bit_Tone2 | bit_Tone1 | - - - - - - - - - - - - - - beep duration - - - - - - - - - - - - - - |
;
sa_F04TABBEG:	RETLW	007H				; Beep code 0x00 - 0 kHz
		RETLW	042H				; Beep code 0x01 - 2 kHz / 0.1 sec
		RETLW	044H				; Beep code 0x02 - 2 kHz / 0.2 sec
		RETLW	048H				; Beep code 0x03 - 2 kHz / 0.5 sec
		RETLW	050H				; Beep code 0x04 - 2 kHz / 1 sec
		RETLW	082H				; Beep code 0x05 - 1 kHz / 0.1 sec
		RETLW	084H				; Beep code 0x06 - 1 kHz / 0.2 sec
		RETLW	088H				; Beep code 0x07 - 1 kHz / 0.5 sec
		RETLW	090H				; Beep code 0x08 - 1 kHz / 1 sec
sa_F04TABEND:	RETLW	041H				; Beep code 0x09 - 2 kHz / 15 msec
;------------------------------------------------------------------------------------------------------------------------
sa_F04EXIT						; FUNCTION 04 delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 05: MODE selector
;		| 7. bit | 6. bit | 5. bit | 4. bit | 3. bit | 2. bit | 1. bit | 0. bit |
; reg_Config	|   x    |   x    |   x    |  CFG3  |   x    |   x    |  CFG2  |  CFG1  |
;
		MOVLW	013H				; MODE changed ?
		ANDWF	PORTA,W				; //
		XORWF	reg_Config,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F05EXIT			; // NO ! - exit function
;
		MOVLW	013H				; // YES ! - update MODE selector from PORTA
		ANDWF	PORTA,W				; //
		MOVWF	reg_Config			; //
;
		BTFSS	reg_Config,000H			; Check CFG1 status
		BSF	bit_SoundOn			; // CFG1 = 0 --> sound ON, CFG1 = 1 --> sound OFF
		BTFSC	reg_Config,000H			; //
		BCF	bit_SoundOn			; //
; Configuration 00 - light effects (free running)
		MOVLW	012H				; CFG2 & CFG3 = 00 ?
		ANDWF	reg_Config,W			; //
		XORLW	000H				; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F0501			; // NO !
		BSF	bit_Function06			; // YES ! - Activate function 06, deactivate other functions
		CLRF	reg_F06Phase			; //
		CLRF	reg_EffectCode			; Initialize light effect parameters
		MOVLW	003H				; //
		MOVWF	reg_EffectSpeed			; //
		BCF	bit_MicIn			; Microphone input inactive
		BCF	bit_Function08			; //
		BCF	bit_Function09			; //
		GOTO	sa_F05EXIT			; Exit function
; Configuration 01 - light effects (sound driven)
sa_F0501:	MOVLW	012H				; CFG2 & CFG3 = 01 ?
		ANDWF	reg_Config,W			; //
		XORLW	002H				; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F0502			; // NO !
		BSF	bit_Function06			; // YES ! - Activate function 06 & 07, deactivate other functions
		CLRF	reg_F06Phase			; //
		CLRF	reg_F07Phase			; //
		CLRF	reg_EffectCode			; Initialize light effect parameters
		MOVLW	003H				; //
		MOVWF	reg_EffectSpeed			; //
		BSF	bit_MicIn			; Microphone input active
		BCF	bit_Function08			; //
		BCF	bit_Function09			; //
		GOTO	sa_F05EXIT			; Exit function
; Configuration 10 - game 1
sa_F0502:	MOVLW	012H				; CFG2 & CFG3 = 10 ?
		ANDWF	reg_Config,W			; //
		XORLW	010H				; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F0503			; // NO !
		BSF	bit_Function08			; // YES ! - Activate function 08, deactivate other functions
		CLRF	reg_F08Phase			; //
		MOVF	reg_Random,W			; //
		MOVWF	reg_GameRnd			; //
		BCF	bit_Function06			; //
		BCF	bit_Function09			; //
		GOTO	sa_F05EXIT			; Exit function
; Configuration 11 - game 2
sa_F0503:	MOVLW	012H				; CFG2 & CFG3 = 11 ?
		ANDWF	reg_Config,W			; //
		XORLW	012H				; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F05EXIT			; // NO ! - exit function
		BSF	bit_Function09			; // YES ! - Activate function 09, deactivate other functions
		CLRF	reg_F09Phase			; //
		BCF	bit_Function06			; //
		BCF	bit_Function08			; //
;------------------------------------------------------------------------------------------------------------------------
sa_F05EXIT						; FUNCTION 05 delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 06: Light effects
		BTFSS	bit_Function06			; Function 06 active
		GOTO	sa_F06EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F06PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg_F06Phase,W			; //
		ADDWF	PCL,F				; //
sa_F06PHASE:	GOTO	sa_F06PHASE0			; //
		GOTO	sa_F06PHASE1			; //
sa_F06PHEND:	GOTO	sa_F06PHASE2			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F06PHASE) - high(sa_F06PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F06PHASE0:	CALL	sa_F06TABLE1			; Read effect start address from the table
		MOVWF	reg_EffectPtr			; //
		CALL	sa_F06TABLE2			; Read effect speed from the table
		MOVWF	reg_EffectLength		; //
		INCF	reg_F06Phase,F			; Phase 1
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06PHASE1:	BTFSS	bit_MicIn			; Microphone input active ?
		GOTO	sa_F06PH101			; // NO !
		BTFSC	port_MIC			; // YES ! - sound detected ?
		GOTO	sa_F06EXIT			; // NO ! - exit function
;
sa_F06PH101:	MOVF	reg_EffectSpeed,W		; Initialize effect step duration
		MOVWF	reg_LongDelay2H			; //
		MOVLW	0FFH				; //
		MOVWF	reg_LongDelay2L			; //
		BCF	bit_LongDelay2			; //
		INCF	reg_EffectPtr,F			; Read first / next step from the table
		CALL	sa_F06TABLE2			; //
		MOVWF	reg_LEDDriver			; //
		MOVLW	009H				; Beep code 0x09
		MOVWF	reg_BeepCode			; //
		BTFSS	bit_MicIn			; Microphone active ?
		BSF	bit_Beep			; // YES ! - skip beep
		INCF	reg_F06Phase,F			; Phase 2
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06PHASE2:	BTFSC	bit_MicIn			; Microphone input active ?
		GOTO	sa_F06PH201			; // YES !
		BTFSC	bit_LongDelay2			; Effect step duration expired ?
		GOTO	sa_F06PH290			; // YES !
		GOTO	sa_F06EXIT			; // NO ! - exit function
;
sa_F06PH201:	BTFSC	port_MIC			; Sound detected ?
		GOTO	sa_F06PH290			; // NO !
		GOTO	sa_F06EXIT			; // YES ! - exit function
;
sa_F06PH290:	MOVLW	000H				; // YES ! - if all steps done, go to phase 0, otherwise go to
		DECFSZ	reg_EffectLength,F		; //  phase 1
		MOVLW	001H				; //
		MOVWF	reg_F06Phase			; //
		GOTO	sa_F06EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F06TABLE1:	MOVLW	high(sa_F06TABBEG1)		; Effect decoder
		MOVWF	PCLATH				; //
		MOVF	reg_EffectCode,W		; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F06TABBEG1) - high(sa_F06TABEND1) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F06TABBEG1:	RETLW	low(sa_EFF00-sa_EFF00)		; Effect code 0x00
		RETLW	low(sa_EFF01-sa_EFF00)		; Effect code 0x01
		RETLW	low(sa_EFF02-sa_EFF00)		; Effect code 0x02
		RETLW	low(sa_EFF03-sa_EFF00)		; Effect code 0x03
		RETLW	low(sa_EFF04-sa_EFF00)		; Effect code 0x04
		RETLW	low(sa_EFF05-sa_EFF00)		; Effect code 0x05
		RETLW	low(sa_EFF06-sa_EFF00)		; Effect code 0x06
		RETLW	low(sa_EFF07-sa_EFF00)		; Effect code 0x07
		RETLW	low(sa_EFF08-sa_EFF00)		; Effect code 0x08
sa_F06TABEND1:	RETLW	low(sa_EFF09-sa_EFF00)		; Effect code 0x09
;------------------------------------------------------------------------------------------------------------------------
;
sa_F06TABLE2:	MOVLW	high(sa_F06TABBEG2)		; Light effect figure table
		MOVWF	PCLATH				; //
		MOVF	reg_EffectPtr,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F06TABBEG2) - high(sa_F06TABEND2) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F06TABBEG2:	
sa_EFF00:	RETLW	002H				; Effect code 0x00, length 0x02
		RETLW	00FH				; //
		RETLW	0F0H				; //
;
sa_EFF01:	RETLW	002H				; Effect code 0x01, length 0x02
		RETLW	055H				; //
		RETLW	0AAH				; //
;
sa_EFF02:	RETLW	002H				; Effect code 0x02, length 0x02
		RETLW	000H				; //
		RETLW	0FFH				; //
;
sa_EFF03:	RETLW	004H				; Effect code 0x03, length 0x04
		RETLW	000H				; //
		RETLW	033H				; //
		RETLW	000H				; //
		RETLW	0CCH				; //
;
sa_EFF04:	RETLW	008H				; Effect code 0x04, length 0x08
		RETLW	000H				; //
		RETLW	0FFH				; //
		RETLW	000H				; //
		RETLW	033H				; //
		RETLW	000H				; //
		RETLW	0FFH				; //
		RETLW	000H				; //
		RETLW	0CCH				; //
;
sa_EFF05:	RETLW	004H				; Effect code 0x05, length 0x04
		RETLW	011H				; //
		RETLW	022H				; //
		RETLW	044H				; //
		RETLW	088H				; //
;
sa_EFF06:	RETLW	008H				; Effect code 0x06, length 0x08
		RETLW	00FH				; //
		RETLW	01EH				; //
		RETLW	03CH				; //
		RETLW	078H				; //
		RETLW	0F0H				; //
		RETLW	0E1H				; //
		RETLW	0C3H				; //
		RETLW	087H				; //

sa_EFF07:	RETLW	008H				; Effect code 0x07, length 0x08
		RETLW	000H				; //
		RETLW	011H				; //
		RETLW	033H				; //
		RETLW	077H				; //
		RETLW	0FFH				; //
		RETLW	0EEH				; //
		RETLW	0CCH				; //
		RETLW	088H				; //
;
sa_EFF08:	RETLW	008H				; Effect code 0x08, length 0x08
		RETLW	000H				; //
		RETLW	018H				; //
		RETLW	03CH				; //
		RETLW	07EH				; //
		RETLW	0FFH				; //
		RETLW	0E7H				; //
		RETLW	0C3H				; //
		RETLW	081H				; //
;
sa_EFF09:	RETLW	010H				; Effect code 0x09, length 0x10
		RETLW	000H				; //
		RETLW	001H				; //
		RETLW	003H				; //
		RETLW	007H				; //
		RETLW	00FH				; //
		RETLW	01FH				; //
		RETLW	03FH				; //
		RETLW	07FH				; //
		RETLW	0FFH				; //
		RETLW	0FEH				; //
		RETLW	0FCH				; //
		RETLW	0F8H				; //
		RETLW	0F0H				; //
		RETLW	0E0H				; //
		RETLW	0C0H				; //
sa_F06TABEND2:	RETLW	080H				; //
;------------------------------------------------------------------------------------------------------------------------
sa_F06EXIT						; FUNCTION 06 delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 07: Key control of light effects
		BTFSS	bit_Function06			; Function 06 active
		GOTO	sa_F07EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F07PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg_F07Phase,W			; //
		ADDWF	PCL,F				; //
sa_F07PHASE:	GOTO	sa_F07PHASE0			; //
sa_F07PHEND:	GOTO	sa_F07PHASE1			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F07PHASE) - high(sa_F07PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F07PHASE0:	BTFSS	bit_Key				; Key pressed ?
		GOTO	sa_F07EXIT			; // NO ! - exit function
;
		MOVLW	001H				; Key code 0x01 ?
		XORWF	reg_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F07PH001			; // NO !
		DECF	reg_EffectSpeed,F		; // YES ! - increase effect speed
		MOVLW	000H				; Maximum speed 0x01 (fastest)
		XORWF	reg_EffectSpeed,W		; //
		MOVLW	001H				; //
		BTFSC	STATUS,Z			; //
		MOVWF	reg_EffectSpeed			; //
		MOVLW	001H				; If minimum speed achieved, select beep code 0x06,
		BTFSC	STATUS,Z			; //  otherwise beep code 0x01
		MOVLW	006H				; //
		GOTO	sa_F07PH099			; //
;
sa_F07PH001:	MOVLW	002H				; Key code 0x02 ?
		XORWF	reg_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F07PH002			; // NO !
		INCF	reg_EffectSpeed,F		; // YES ! - decrease effect speed
		MOVLW	006H				; Minimum speed 0x05 (slowest)
		XORWF	reg_EffectSpeed,W		; //
		MOVLW	005H				; //
		BTFSC	STATUS,Z			; //
		MOVWF	reg_EffectSpeed			; //
		MOVLW	001H				; If maximum speed achieved, select beep code 0x06,
		BTFSC	STATUS,Z			; //  otherwise beep code 0x01
		MOVLW	006H				; //
		GOTO	sa_F07PH099			; //
;
sa_F07PH002:	MOVLW	004H				; Key code 0x04 ?
		XORWF	reg_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F07PH003			; // NO !
		INCF	reg_EffectCode,F		; // YES ! - next effect
		MOVLW	00AH				; // maximum effect code 0x09
		XORWF	reg_EffectCode,W		; //
		MOVLW	000H				; //
		BTFSC	STATUS,Z			; //
		MOVWF	reg_EffectCode			; //
		CLRF	reg_F06Phase			; Restart effect
		MOVLW	001H				; Beep code 0x01
		GOTO	sa_F07PH099			; //
;
sa_F07PH003:	MOVLW	008H				; Key code 0x08 ?
		XORWF	reg_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F07PH099			; // NO !
		DECF	reg_EffectCode,F		; // YES ! - previous effect
		MOVLW	0FFH				; //
		XORWF	reg_EffectCode,W		; //
		MOVLW	009H				; // Maximum effect code 0x09
		BTFSC	STATUS,Z			; //
		MOVWF	reg_EffectCode			; //
		CLRF	reg_F06Phase			; Restart effect
		MOVLW	001H				; Beep code 0x01
		GOTO	sa_F07PH099			; //
;
sa_F07PH099:	MOVWF	reg_BeepCode			; Store beep code
		BSF	bit_Beep			; Request beep
		INCF	reg_F07Phase,F			; Phase 1
		GOTO	sa_F07EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F07PHASE1:	BTFSS	bit_Key				; Key released ?
		CLRF	reg_F07Phase			; // YES ! - phase 0
		GOTO	sa_F07EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F07EXIT						; FUNCTION 07 delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 08: Game 1
		BTFSS	bit_Function08			; Function 08 active
		GOTO	sa_F08EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F08PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg_F08Phase,W			; //
		ADDWF	PCL,F				; //
sa_F08PHASE:	GOTO	sa_F08PHASE0			; //
		GOTO	sa_F08PHASE1			; //
		GOTO	sa_F08PHASE2			; //
		GOTO	sa_F08PHASE3			; //
		GOTO	sa_F08PHASE4			; //
		GOTO	sa_F08PHASE5			; //
		GOTO	sa_F08PHASE6			; //
		GOTO	sa_F08PHASE7			; //
sa_F08PHEND:	GOTO	sa_F08PHASE8			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F08PHASE) - high(sa_F08PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; Demo mode
sa_F08PHASE0:	MOVF	reg_Random,W			; Ensures, that two consecutive pseudo-random values are
		XORWF	reg_GameRnd,W			; // different
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08EXIT			; Exit function
		MOVF	reg_Random,W			; Store new pseudo-random valu
		MOVWF	reg_GameRnd			; //
;
		CALL	sa_F08TABLE1			; Get random figure
		MOVWF	reg_LEDDriver			; //
		MOVLW	006H				; Initialize demo speed
		MOVWF	reg_LongDelay2H			; //
		MOVLW	0FFH				; //
		MOVWF	reg_LongDelay2L			; //
		BCF	bit_LongDelay2			; //
		INCF	reg_F08Phase,F			; Phase 1
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE1:	BTFSC	bit_Key				; Any key pressed ?
		GOTO	sa_F08PH101			; // YES ! - start game
		BTFSC	bit_LongDelay2			; Figure period expired
		CLRF	reg_F08Phase			; // YES ! - phase 0
		GOTO	sa_F08EXIT			; // NO ! - exit function
; Select game level
sa_F08PH101:	CLRF	reg_GameLevel			; Initialize game level
		MOVLW	002H				; Key code 0x02 ?
		XORWF	reg_KeyCode,W			; //
		MOVLW	000H				; // game level 1
		BTFSC	STATUS,Z			; //
		MOVWF	reg_GameLevel			; // YES !
;
		MOVLW	008H				; Key code 0x08 ?
		XORWF	reg_KeyCode,W			; //
		MOVLW	001H				; // game level 2
		BTFSC	STATUS,Z			; //
		MOVWF	reg_GameLevel			; // YES !
;
		MOVLW	004H				; Key code 0x04 ?
		XORWF	reg_KeyCode,W			; //
		MOVLW	002H				; // game level 3
		BTFSC	STATUS,Z			; //
		MOVWF	reg_GameLevel			; // YES !
;
		MOVLW	001H				; Key code 0x01 ?
		XORWF	reg_KeyCode,W			; //
		MOVLW	003H				; // game level 4
		BTFSC	STATUS,Z			; //
		MOVWF	reg_GameLevel			; // YES !
;
		INCF	reg_F08Phase,F			; Phase 2
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; Game mode
sa_F08PHASE2:	MOVLW	000H				; Get level figure
		CALL	sa_F08TABLE2			; //
		MOVWF	reg_LEDDriver			; //
		MOVLW	005H				; Get speed
		CALL	sa_F08TABLE2			; //
		MOVWF	reg_GameSpeed			; //
		MOVLW	00AH				; Get step count
		CALL	sa_F08TABLE2			; //
		MOVWF	reg_GameStep			; //
		MOVLW	004H				; Request beep code 0x04
		MOVWF	reg_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	016H				; Initialize level figure duration for 1.5 sec
		MOVWF	reg_LongDelay2H			; //
		MOVLW	0FFH				; //
		MOVWF	reg_LongDelay2L			; //
		BCF	bit_LongDelay2			; //
		INCF	reg_F08Phase,F			; Phase 3
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE3:	BTFSC	bit_LongDelay2			; Level figure duration expired ?
		INCF	reg_F08Phase,F			; // YES ! - phase 4
		GOTO	sa_F08EXIT			; // NO ! - exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE4:	MOVF	reg_Random,W			; Ensures, that two consecutive pseudo-random values are
		XORWF	reg_GameRnd,W			; // different
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08EXIT			; Exit function
		MOVF	reg_Random,W			; Store new pseudo-random value
		MOVWF	reg_GameRnd			; //
;
		CALL	sa_F08TABLE1			; Get random figure
		MOVWF	reg_LEDDriver			; //
		MOVF	reg_GameSpeed,W			; Initialize figure duration
		MOVWF	reg_LongDelay2H			; //
		MOVLW	0FFH				; //
		MOVWF	reg_LongDelay2L			; //
		BCF	bit_LongDelay2			; //
		MOVLW	009H				; Request beep code 0x09
		MOVWF	reg_BeepCode			; //
		BSF	bit_Beep			; //
		INCF	reg_F08Phase,F			; Phase 5
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE5:	BTFSC	bit_LongDelay2			; Figure duration expired ?
		GOTO	sa_F08PH501			; // YES !
		BTFSS	bit_Key				; Key released ?
		INCF	reg_F08Phase,F			; // YES ! - phase 6
		GOTO	sa_F08EXIT			; Exit function
;
sa_F08PH501:	MOVLW	0FFH				; Time-out
		MOVWF	reg_LEDDriver			; //
		MOVLW	008H				; Request beep code 0x08
		MOVWF	reg_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	007H				; Phase 7
		MOVWF	reg_F08Phase			; //
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE6:	BTFSC	bit_LongDelay2			; Figure duration expired ?
		GOTO	sa_F08PH601			; // YES !
		BTFSC	bit_Key				; Key pressed ?
		GOTO	sa_F08PH602			; // YES !
		GOTO	sa_F08EXIT			; // NO ! - exit function
;
sa_F08PH601:	MOVLW	0FFH				; Time-out
		MOVWF	reg_LEDDriver			; //
		MOVLW	008H				; Request beep code 0x08
		MOVWF	reg_BeepCode			; //
		BSF	bit_Beep			; //
		INCF	reg_F08Phase,F			; Phase 7
		GOTO	sa_F08EXIT			; Exit function
;
sa_F08PH602:	MOVLW	001H				; Key code 0x01 ?
		XORWF	reg_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F08PH603			; // NO !
		MOVLW	0C0H				; Key & figure matching ?
		XORWF	reg_LEDDriver,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F08PH601			; // NO !
		GOTO	sa_F08PH610			; // YES !
;
sa_F08PH603:	MOVLW	002H				; Key code 0x02 ?
		XORWF	reg_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F08PH604			; // NO !
		MOVLW	003H				; Key & figure matching ?
		XORWF	reg_LEDDriver,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F08PH601			; // NO !
		GOTO	sa_F08PH610			; // YES !
;
sa_F08PH604:	MOVLW	004H				; Key code 0x04 ?
		XORWF	reg_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F08PH605			; // NO !
		MOVLW	030H				; Key & figure matching ?
		XORWF	reg_LEDDriver,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F08PH601			; // NO !
		GOTO	sa_F08PH610			; // YES !
;
sa_F08PH605:	MOVLW	008H				; Key code 0x08 ?
		XORWF	reg_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F08PH601			; // NO !
		MOVLW	00CH				; Key & figure matching ?
		XORWF	reg_LEDDriver,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F08PH601			; // NO !
;		GOTO	sa_F08PH610			; // YES !
;
sa_F08PH610:	MOVLW	008H				; Phase 8
		MOVWF	reg_F08Phase			; //
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE7:	BTFSC	bit_Key				; Key released ?
		GOTO	sa_F08EXIT			; // NO ! - exit function
		BTFSS	bit_Beep			; Beep finished ?
		CLRF	reg_F08Phase			; // YES ! - restart function
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08PHASE8:	BTFSS	bit_LongDelay2			; Figure duration expired ?
		GOTO	sa_F08EXIT			; // NO ! - exit function
;
		MOVLW	004H				; All steps done ? (phase 4)
		MOVWF	reg_F08Phase			; //
		DECFSZ	reg_GameStep,F			; //
		GOTO	sa_F08EXIT			; // NO ! - go to phase 4
;
		MOVLW	con_Game1MaxLevel		; Last game level ?
		XORWF	reg_GameLevel,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F08EXIT			; // YES ! - go to phase 4, no level change
		INCF	reg_GameLevel,F			; Increment game level
		MOVLW	002H				; Phase 2
		MOVWF	reg_F08Phase			; //
		GOTO	sa_F08EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F08TABLE1:	MOVLW	high(sa_F08TABBEG1)		; Random figure selection
		MOVWF	PCLATH				; //
		MOVF	reg_GameRnd,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F08TABBEG1) - high(sa_F08TABEND1) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F08TABBEG1:	RETLW	003H				; Figure code 0x00
		RETLW	00CH				; Figure code 0x01
		RETLW	030H				; Figure code 0x02
sa_F08TABEND1:	RETLW	0C0H				; Figure code 0x03
;------------------------------------------------------------------------------------------------------------------------
; Table offset Wreg = 0x00 --> get level figure /  Wreg = 0x05 --> get speed /  Wreg = 0x0A --> get step count
sa_F08TABLE2:	MOVWF	reg_Temp1			; Get game parameters
		MOVLW	high(sa_F08TABBEG2)		; //
		MOVWF	PCLATH				; //
		MOVF	reg_Temp1,W			; //
		ADDWF	reg_GameLevel,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F08TABBEG2) - high(sa_F08TABEND2) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; Level figure
sa_F08TABBEG2:	RETLW	020H				; Level 1 - figure
		RETLW	060H				; Level 2 - figure
		RETLW	070H				; Level 3 - figure
		RETLW	0F0H				; Level 4 - figure
		RETLW	0F8H				; Level 5 - figure
; Level speed
		RETLW	013H				; Level 1 - speed
		RETLW	00FH				; Level 2 - speed
		RETLW	00BH				; Level 3 - speed
		RETLW	008H				; Level 4 - speed
		RETLW	006H				; Level 5 - speed
; Level step count
		RETLW	00AH				; Level 1 - step count (10)
		RETLW	00AH				; Level 2 - step count (10)
		RETLW	014H				; Level 3 - step count (20)
		RETLW	01EH				; Level 4 - step count (30)
sa_F08TABEND2:	RETLW	005H				; Level 5 - step count (unlimited)
;------------------------------------------------------------------------------------------------------------------------
sa_F08EXIT						; FUNCTION 08 delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
; FUNCTION 09: Game 2
		BTFSS	bit_Function09			; Function 09 active
		GOTO	sa_F09EXIT			; // NO ! - skip function
;
		MOVLW	high(sa_F09PHASE)		; Function phase switch
		MOVWF	PCLATH				; //
		MOVF	reg_F09Phase,W			; //
		ADDWF	PCL,F				; //
sa_F09PHASE:	GOTO	sa_F09PHASE00			; //
		GOTO	sa_F09PHASE01			; //
		GOTO	sa_F09PHASE02			; //
		GOTO	sa_F09PHASE03			; //
		GOTO	sa_F09PHASE04			; //
		GOTO	sa_F09PHASE05			; //
		GOTO	sa_F09PHASE06			; //
		GOTO	sa_F09PHASE07			; //
		GOTO	sa_F09PHASE08			; //
		GOTO	sa_F09PHASE09			; //
		GOTO	sa_F09PHASE10			; //
		GOTO	sa_F09PHASE11			; //
sa_F09PHEND:	GOTO	sa_F09PHASE12			; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F09PHASE) - high(sa_F09PHEND) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; Demo mode
sa_F09PHASE00:	CALL	sa_F09TABLE1			; Get random figure
		MOVWF	reg_LEDDriver			; //
		MOVLW	007H				; Initialize demo figure duration
		MOVWF	reg_LongDelay2H			; //
		MOVLW	0FFH				; //
		MOVWF	reg_LongDelay2L			; //
		BCF	bit_LongDelay2			; //
		INCF	reg_F09Phase,F			; Phase 1
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE01:	BTFSC	bit_Key				; Any key pressed ?
		GOTO	sa_F09PH0301			; // YES ! - start game (go directly to phase 3)
		BTFSC	bit_LongDelay2			; Figure period expired
		INCF	reg_F09Phase,F			; // YES ! - phase 2
		GOTO	sa_F09EXIT			; // NO ! - exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE02:	CLRF	reg_LEDDriver			; Turn-off all LEDs
		MOVLW	004H				; Initialize demo figure duration
		MOVWF	reg_LongDelay2H			; //
		MOVLW	0FFH				; //
		MOVWF	reg_LongDelay2L			; //
		BCF	bit_LongDelay2			; //
		INCF	reg_F09Phase,F			; Phase 3
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE03:	BTFSC	bit_Key				; Any key pressed ?
		GOTO	sa_F09PH0301			; // YES ! - start game
		BTFSC	bit_LongDelay2			; Figure period expired
		CLRF	reg_F09Phase			; // YES ! - phase 0
		GOTO	sa_F09EXIT			; // NO ! - exit function
;
sa_F09PH0301:	CLRF	reg_GameLevel			; Initialize game level
		MOVLW	002H				; Key code 0x02 ?
		XORWF	reg_KeyCode,W			; //
		MOVLW	000H				; // game level 1
		BTFSC	STATUS,Z			; //
		MOVWF	reg_GameLevel			; // YES !
;
		MOVLW	008H				; Key code 0x08 ?
		XORWF	reg_KeyCode,W			; //
		MOVLW	001H				; // game level 2
		BTFSC	STATUS,Z			; //
		MOVWF	reg_GameLevel			; // YES !
;
		MOVLW	004H				; Key code 0x04 ?
		XORWF	reg_KeyCode,W			; //
		MOVLW	002H				; // game level 3
		BTFSC	STATUS,Z			; //
		MOVWF	reg_GameLevel			; // YES !
;
		MOVLW	001H				; Key code 0x01 ?
		XORWF	reg_KeyCode,W			; //
		MOVLW	003H				; // game level 4
		BTFSC	STATUS,Z			; //
		MOVWF	reg_GameLevel			; // YES !
;
		MOVLW	004H				; Phase 4
		MOVWF	reg_F09Phase			; //
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
; Game mode
sa_F09PHASE04:	MOVLW	000H				; Get level figure
		CALL	sa_F09TABLE2			; //
		MOVWF	reg_LEDDriver			; //
		MOVLW	00EH				; Get repeat count
		CALL	sa_F09TABLE2			; //
		MOVWF	reg_GameRepeat			; //
		MOVLW	004H				; Request beep code 0x04
		MOVWF	reg_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	016H				; Initialize level figure duration for 1.5 sec
		MOVWF	reg_LongDelay2H			; //
		MOVLW	0FFH				; //
		MOVWF	reg_LongDelay2L			; //
		BCF	bit_LongDelay2			; //
		INCF	reg_F09Phase,F			; Phase 5
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE05:	BTFSS	bit_LongDelay2			; Level figure duration expired ?
		GOTO	sa_F09EXIT			; // NO ! - exit function
;
		MOVLW	007H				; Get step count
		CALL	sa_F09TABLE2			; //
		MOVWF	reg_GameStep			; //
		CLRF	reg_GameBuf1			; Clear buffer
		CLRF	reg_GameBuf2			; //
		INCF	reg_F09Phase,F			; Phase 6
		GOTO	sa_F09EXIT			; // NO ! - exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE06:	BCF	STATUS,C			; Store random value the buffer
		RLF	reg_GameBuf1,F			; //
		RLF	reg_GameBuf2,F			; //
		RLF	reg_GameBuf1,F			; //
		RLF	reg_GameBuf2,F			; //
		MOVF	reg_Random,W			; //
		IORWF	reg_GameBuf1,F			; //
		CALL	sa_F09TABLE1			; Get random figure
		MOVWF	reg_LEDDriver			; //
		MOVLW	007H				; Initialize figure duration
		MOVWF	reg_LongDelay2H			; //
		MOVLW	0FFH				; //
		MOVWF	reg_LongDelay2L			; //
		BCF	bit_LongDelay2			; //
		MOVLW	009H				; Request beep code 0x09
		MOVWF	reg_BeepCode			; //
		BSF	bit_Beep			; //
		INCF	reg_F09Phase,F			; Phase 7
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE07:	BTFSC	bit_LongDelay2			; Figure period expired ?
		INCF	reg_F09Phase,F			; // YES ! - phase 8
		GOTO	sa_F09EXIT			; // NO ! - exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE08:	CLRF	reg_LEDDriver			; Turn-off all LEDs
		MOVLW	004H				; Initialize figure duration
		MOVWF	reg_LongDelay2H			; //
		MOVLW	0FFH				; //
		MOVWF	reg_LongDelay2L			; //
		BCF	bit_LongDelay2			; //
		INCF	reg_F09Phase,F			; Phase 9
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE09:	BTFSS	bit_LongDelay2			; Figure period expired ?
		GOTO	sa_F09EXIT			; // NO ! - exit function
		DECFSZ	reg_GameStep,F			; All steps done ?
		GOTO	sa_F09PH0990			; // NO !
;
		MOVLW	007H				; // YES ! - get step count
		CALL	sa_F09TABLE2			; //
		MOVWF	reg_GameStep			; //
;
		MOVLW	008H				; Align buffer
		SUBWF	reg_GameStep,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F09PH0902			; //
		XORLW	0FFH				; // two's complement
		ADDLW	001H				; //
		MOVWF	reg_Temp1			; //
sa_F09PH0901:	RLF	reg_GameBuf1,F			; //
		RLF	reg_GameBuf2,F			; //
		RLF	reg_GameBuf1,F			; //
		RLF	reg_GameBuf2,F			; //
		DECFSZ	reg_Temp1,F			; //
		GOTO	sa_F09PH0901			; //
sa_F09PH0902:	MOVLW	00AH				; Phase 10
		MOVWF	reg_F09Phase			; //
		GOTO	sa_F09EXIT			; Exit function
;
sa_F09PH0990:	MOVLW	006H				; Phase 6
		MOVWF	reg_F09Phase			; //
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE10:	BTFSS	bit_Key				; Key pressed ?
		GOTO	sa_F09EXIT			; // NO ! - exit function
;
		RLF	reg_GameBuf1,F			; // YES ! - get random code from buffer
		RLF	reg_GameBuf2,F			; //
		RLF	reg_Temp1,F			; //
		RLF	reg_GameBuf1,F			; //
		RLF	reg_GameBuf2,F			; //
		RLF	reg_Temp1,F			; //
		MOVLW	003H				; //
		ANDWF	reg_Temp1,F			; //
;		
sa_F09PH1010:	MOVLW	001H				; Key code 0x01 ?
		XORWF	reg_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH1011			; // NO !
		MOVLW	003H				; Key & buffer matching ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH1090			; // NO !
		MOVLW	0C0H				; // YES ! - display match figure
		GOTO	sa_F09PH1014			; //
;
sa_F09PH1011:	MOVLW	002H				; Key code 0x02 ?
		XORWF	reg_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH1012			; // NO !
		MOVLW	000H				; Key & buffer matching ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH1090			; // NO !
		MOVLW	003H				; // YES ! - display match figure
		GOTO	sa_F09PH1014			; //
;
sa_F09PH1012:	MOVLW	004H				; Key code 0x04 ?
		XORWF	reg_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH1013			; // NO !
		MOVLW	002H				; Key & buffer matching ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH1090			; // NO !
		MOVLW	030H				; // YES ! - display match figure
		GOTO	sa_F09PH1014			; //
;
sa_F09PH1013:	MOVLW	008H				; Key code 0x08 ?
		XORWF	reg_KeyCode,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH1090			; // NO !
		MOVLW	001H				; Key & buffer matching ?
		XORWF	reg_Temp1,W			; //
		BTFSS	STATUS,Z			; //
		GOTO	sa_F09PH1090			; // NO !
		MOVLW	00CH				; // YES ! - display match figure
;		GOTO	sa_F09PH1014			; //
;
sa_F09PH1014:	MOVWF	reg_LEDDriver			; Display match figure
		MOVLW	009H				; Request beep code 0x09
		MOVWF	reg_BeepCode			; //
		BSF	bit_Beep			; //
		INCF	reg_F09Phase,F			; Phase 11
		GOTO	sa_F09EXIT			; Exit function
;
sa_F09PH1090:	MOVLW	0FFH				; Key & buffer mismatch figure
		MOVWF	reg_LEDDriver			; //
		MOVLW	008H				; Request beep code 0x08
		MOVWF	reg_BeepCode			; //
		BSF	bit_Beep			; //
		MOVLW	00CH				; Phase 12
		MOVWF	reg_F09Phase			; //
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE11:	BTFSC	bit_Key				; Key released ?
		GOTO	sa_F09EXIT			; // NO ! - exit function
		CLRF	reg_LEDDriver			; // YES ! - clear match figure
		DECF	reg_F09Phase,F			; All steps processed ?
		DECFSZ	reg_GameStep,F			; //
		GOTO	sa_F09EXIT			; // NO ! - go to phase 10

		MOVLW	009H				; // YES ! - delay before next sequence starts
		MOVWF	reg_LongDelay2H			; //
		MOVLW	0FFH				; //
		MOVWF	reg_LongDelay2L			; //
		BCF	bit_LongDelay2			; //
;
		MOVLW	005H				; Level finished ? (Set phase 5)
		MOVWF	reg_F09Phase			; //
		DECFSZ	reg_GameRepeat,F		; //
		GOTO	sa_F09EXIT			; // NO ! - go to phase 5
;
		MOVLW	con_Game2MaxLevel		; Last game level ?
		XORWF	reg_GameLevel,W			; //
		BTFSC	STATUS,Z			; //
		GOTO	sa_F09EXIT			; // YES ! - go to phase 5 (no level change)
		INCF	reg_GameLevel,F			; // NO ! - increment game level
		MOVLW	004H				; Phase 4
		MOVWF	reg_F09Phase			; //
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09PHASE12:	BTFSC	bit_Key				; Key released ?
		GOTO	sa_F09EXIT			; // NO ! - exit function
		BTFSS	bit_Beep			; Beep finished ?
		CLRF	reg_F09Phase			; // YES ! - restart function
		GOTO	sa_F09EXIT			; Exit function
;------------------------------------------------------------------------------------------------------------------------
sa_F09TABLE1:	MOVLW	high(sa_F09TABBEG1)		; Random figure selection
		MOVWF	PCLATH				; //
		MOVF	reg_Random,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F09TABBEG1) - high(sa_F09TABEND1) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
sa_F09TABBEG1:	RETLW	003H				; Figure code 0x00
		RETLW	00CH				; Figure code 0x01
		RETLW	030H				; Figure code 0x02
sa_F09TABEND1:	RETLW	0C0H				; Figure code 0x03
;------------------------------------------------------------------------------------------------------------------------
; Table offset Wreg = 0x00 --> get level figure /  Wreg = 0x07 --> get step count /  Wreg = 0x0E --> get repeat count
sa_F09TABLE2:	MOVWF	reg_Temp1			; Get game parameters
		MOVLW	high(sa_F09TABBEG2)		; //
		MOVWF	PCLATH				; //
		MOVF	reg_Temp1,W			; //
		ADDWF	reg_GameLevel,W			; //
		ADDWF	PCL,F				; //
;****** MPASM compiler will issue an error message because of PCLATH overrun !!! ****************************************
	IF	high(sa_F09TABBEG2) - high(sa_F09TABEND2) != 0
		ERROR	"The 256B-boundary of program memory exceeding not allowed !!!"
	ENDIF
;************************************************************************************************************************
; Level figure
sa_F09TABBEG2:	RETLW	020H				; Level 1 - figure
		RETLW	060H				; Level 2 - figure
		RETLW	070H				; Level 3 - figure
		RETLW	0F0H				; Level 4 - figure
		RETLW	0F8H				; Level 5 - figure
		RETLW	0F9H				; Level 6 - figure
		RETLW	0FDH				; Level 7 - figure
; Level step count
		RETLW	002H				; Level 1 - step count
		RETLW	003H				; Level 2 - step count
		RETLW	004H				; Level 3 - step count
		RETLW	005H				; Level 4 - step count
		RETLW	006H				; Level 5 - step count
		RETLW	007H				; Level 6 - step count
		RETLW	008H				; Level 7 - step count
; Level repeat count
		RETLW	003H				; Level 1 - repeat count
		RETLW	003H				; Level 2 - repeat count
		RETLW	003H				; Level 3 - repeat count
		RETLW	003H				; Level 4 - repeat count
		RETLW	003H				; Level 5 - repeat count
		RETLW	003H				; Level 6 - repeat count
sa_F09TABEND2:	RETLW	003H				; Level 7 - repeat count (unlimited)
;------------------------------------------------------------------------------------------------------------------------
sa_F09EXIT						; FUNCTION 09 delimiter
;////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
;
		GOTO	sa_MAINLOOP			; Back to mainloop begin
;
;************************************************************************************************************************
;*		A P P L I C A T I O N     E N D										*
;************************************************************************************************************************
;
;
;
		END
;
; component layout
;
;	--------------------------------------------------
;	|  S203                                    S200  |
;	| (0x01)        LD202        LD201        (0x04) |
;	|                                                |
;	|                                                |
;	|   LD203                                LD200   |
;	|                                                |
;	|                                                |
;	|                                                |
;	|   LD204                                LD207   |
;	|                                                |
;	|                                                |
;	| (0x02)        LD205        LD206        (0x08) |
;	|  S202                                    S201  |
;	|                                                |
;	|                                                |
;	|                                                |
;
;
;	      bit       |   7.  |   6.  |   5.  |   4.  |   3.  |   2.  |   1.  |   0.  |
;	---------------------------------------------------------------------------------
;	reg_LEDDriver   | LD203 | LD202 | LD201 | LD200 | LD207 | LD206 | LD205 | LD204 |


